@@ -23,6 +23,7 @@
 __version__ = "$Id: 0.py 2265 2010-02-21 19:16:26Z v13 $"
 
 import gtk
+import glib
 import hildon
 from hildondesktop import StatusMenuItem
 import subprocess
@@ -43,7 +44,8 @@ def getButton():
 
 class Activities:
     def __init__(self):
-	pass
+	self.cb=None
+	self.cb_data=None
 
     def getActivities(self):
 	r=self.run("list")
@@ -62,13 +64,27 @@ class Activities:
 
     def run(self, *args):
 	cmd=[ACT] + list(args)
-#	print "cmd:", cmd
 	c0=subprocess.Popen(cmd, stdout=subprocess.PIPE)
 	r=c0.communicate()
 	ret=r[0]
-#	print "ret:", ret
 	return(ret)
 
+    def run_async(self, cb, data, *args):
+	if self.cb!=None:
+	    print "Attempt to spawn a second child!"
+
+	cmd=[ACT] + list(args)
+	r=glib.spawn_async(cmd, flags=glib.SPAWN_DO_NOT_REAP_CHILD)
+	pid=r[0]
+	self.cb=cb
+	self.cb_data=data
+	glib.child_watch_add(pid, self.child_end)
+
+    def child_end(self, pid, condition):
+	self.cb(pid, condition, self.cb_data)
+	self.cb=None
+	self.cb_data=None
+	#glib.spawn_close_pid(pid)
 
 class MainWin(hildon.StackableWindow, Activities):
     def __init__(self):
@@ -130,16 +146,37 @@ class MainWin(hildon.StackableWindow, Activities):
 	except:
 	    pass
 
+    def showWaitDialog(self, txt):
+	dialog=gtk.Dialog("Please wait", self,
+	    gtk.DIALOG_DESTROY_WITH_PARENT, buttons=())
+	msg="Switching to desktop activity \"%s\"" % txt
+	label=gtk.Label(msg)
+	dialog.vbox.pack_start(label)
+	dialog.show_all()
+
+	# This way the dialog cannot be closed
+	dialog.connect("delete-event", self.doNothing, None)
+
+	self.wait_dialog=dialog
+
+    def doNothing(self, dlg, event, dt):
+	return(True)
+
+    def closeWaitDialog(self):
+	self.wait_dialog.destroy()
 
     def signalSelect(self, dt):
 #	print "select"
 	txt=self.sel.get_current_text()
 
 	# We cannot provide feedback because it is not shown
-	self.run("change", txt)
+	self.run_async(self.signalSelectDone, None, "change", txt)
+	
+	self.showWaitDialog(txt)
 
+    def signalSelectDone(self, pid, condition, data):
 	self.showCurrent()
-
+	self.closeWaitDialog()
 	self.destroy()
 
     def signalNew(self, dt):
@@ -148,13 +185,13 @@ class MainWin(hildon.StackableWindow, Activities):
 	dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
 	dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
 
-	label=gtk.Label("""Enter the name of the new activity.
+	label=gtk.Label("""Enter the name of the new desktop activity.
 It will be a clone of the current one.""")
 	label.set_alignment(0,0)
 	dialog.vbox.pack_start(label)
 
 	entry=hildon.Entry(gtk.HILDON_SIZE_AUTO)
-	entry.set_placeholder("Activity name")
+	entry.set_placeholder("Desktop activity name")
 	dialog.vbox.pack_start(entry)
 	
 	doit=True
@@ -179,14 +216,20 @@ It will be a clone of the current one.""")
 	    # already stored current settings. Just call "load"
 	    self.run("store", cur)
 	    self.run("clone", cur, txt)
-	    self.run("load", txt)
-	    dialog.destroy()
-	    self.showCurrent();
+	    self.run_async(self.signalNewDone, dialog, "load", txt)
+	    self.showWaitDialog(txt)
 	else:
 	    dialog.destroy()
+	    self.refresh_list()
 
+    def signalNewDone(self, pid, condition, data):
+	dialog=data
+
+	self.closeWaitDialog()
+	dialog.destroy()
+	self.showCurrent();
 	self.refresh_list()
-
+	
     def signalRename(self, dt):
 	old=self.sel.get_current_text()
 
@@ -195,13 +238,14 @@ It will be a clone of the current one.""")
 	dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
 	dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
 
-	print "enter"
-	label=gtk.Label("Enter the new name of the activity \"%s\"." % old)
+	#print "enter"
+	label=gtk.Label(
+	    "Enter the new name of the desktop activity \"%s\"." % old)
 	label.set_alignment(0,0)
 	dialog.vbox.pack_start(label)
 
 	entry=hildon.Entry(gtk.HILDON_SIZE_AUTO)
-	entry.set_placeholder("New activity name")
+	entry.set_placeholder("New desktop activity name")
 	dialog.vbox.pack_start(entry)
 
 	dialog.show_all()
@@ -214,8 +258,8 @@ It will be a clone of the current one.""")
 
 	dialog.destroy()
 
-	print "old:", old
-	print "txt:", txt
+	#print "old:", old
+	#print "txt:", txt
 	if doit:
 	    self.run("rename", old, txt)
 	    self.refresh_list()
@@ -263,17 +307,22 @@ It will be a clone of the current one.""")
 	d.vbox.pack_start(pa)
 
 	l=gtk.Label("""\
-This is ActMan v%s. A desktop activity manager for N900.
+This is Desktop Activity Manager (ActMan) v%s.
 
-An activity is the set of all desktop Widgets, Bookmarks and Shortcuts along with the desktop background.
+A desktop activity is the set of all desktop Widgets, Bookmarks and Shortcuts along with the desktop background.
 
-ActMon allows the user to have multiple such activities for different use cases.
-
-This application can be distributed under the terms of GPLv3 license.
+ActMan allows the user to have multiple such activities for different use cases.
 
 Copyright (c) 2010 Harhalakis Stefanos <v13@v13.gr>.
 
 Send comments and bug reports to the above address.
+
+This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 """ % (ver, )
 	    )
 	l.set_line_wrap(True)
@@ -342,6 +391,10 @@ class actman_applet(StatusMenuItem):
 	self.but.connect("clicked", self.eventButClicked)
 	self.but.set_alignment(0.2,0.5,1,1)
 
+	img=gtk.image_new_from_icon_name("actman-applet", gtk.ICON_SIZE_BUTTON)
+	self.but.set_image(img)
+	self.but.set_image_position(gtk.POS_LEFT)
+
 	self.add(self.but)
 	self.show_all()
 
